# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""SQL-specific TextView

This module provides a SQL-specific TextView widget. Depending on
your system it is a gtksourceview2 or gtksourceview implementation.

Both implementations inherit ``SQLViewBase`` which connects to the
configuration instance to update some options when they are changed
through the preferences dialog.

Example usage
=============

    .. sourcecode:: python

        >>> from cf.ui.widgets.sqlview import SQLView
        >>> import gtk
        >>> w = gtk.Window()
        >>> sw = gtk.ScrolledWindow()
        >>> view = SQLView(app)
        >>> view.get_buffer().set_text("select * from foo")
        >>> sw.add(view)
        >>> w.add(sw)
        >>> w.show_all()

"""

import gconf
import gtk
import gtksourceview2
import gobject
import pango

from cf import sqlparse


class SQLView(gtksourceview2.View):
    """SQLViewBase implementation"""

    def __init__(self, app, editor=None):
        gtksourceview2.View.__init__(self)
        self.editor = editor
        self.buffer = gtksourceview2.Buffer()
        self.set_buffer(self.buffer)
        self.set_show_line_marks(True)
        pixbuf = self.render_icon(gtk.STOCK_GO_FORWARD,
                                  gtk.ICON_SIZE_MENU)
        width = pixbuf.get_width()
        height = pixbuf.get_height()
        pixbuf = pixbuf.scale_simple(width/2, height/2,
                                     gtk.gdk.INTERP_BILINEAR)
        self.set_mark_category_pixbuf('sql', pixbuf)
        lang_manager = gtksourceview2.language_manager_get_default()
        self.buffer.set_language(lang_manager.get_language('sql'))
        self.app = app
        self.app.config.connect('changed', self.on_config_changed)
        self.update_textview_options()
        self._buffer_changed_cb = None
        self.buffer.connect('changed', self.on_buffer_changed)
        if self.editor is not None:
            self.editor.connect('connection-changed', lambda e, c:
                                self.on_buffer_changed(self.buffer))
        self._sql_marks = []

    def on_buffer_changed(self, buffer):
        """Installs timeout callback to self.buffer_changed_cb()."""
        if self._buffer_changed_cb is not None:
            gobject.source_remove(self._buffer_changed_cb)
            self._buffer_changed_cb = None
        self._buffer_changed_cb = gobject.timeout_add(500,
                                                      self.buffer_changed_cb,
                                                      buffer)

    def on_config_changed(self, config, option, value):
        """Updates view and buffer on configuration change."""
        if option.startswith('editor.'):
            gobject.idle_add(self.update_textview_options)
        if option == "sqlparse.enabled":
            gobject.idle_add(self.buffer_changed_cb, self.buffer)

    def buffer_changed_cb(self, buffer):
        """Update marks."""
        # TODO(andi): Needs optimizations.
        while self._sql_marks:
            mark = self._sql_marks.pop()
            self.buffer.delete_mark(mark)
        if self.app.config.get("sqlparse.enabled", True):
            for iter_start, iter_end in self.get_statements():
                mark = buffer.create_source_mark(None, 'sql', iter_start)
                mark.set_visible(True)
                self._sql_marks.append(mark)
        self._buffer_changed_cb = None
        return False

    def update_textview_options(self):
        c = self.app.config
        buf = self.get_buffer()
        if c.get('editor.wrap_text'):
            if c.get('editor.wrap_split'):
                self.set_wrap_mode(gtk.WRAP_CHAR)
            else:
                self.set_wrap_mode(gtk.WRAP_WORD)
        else:
            self.set_wrap_mode(gtk.WRAP_NONE)
        self.set_show_line_numbers(c.get('editor.display_line_numbers'))
        self.set_highlight_current_line(c.get('editor.highlight_current_line'))
        self.set_insert_spaces_instead_of_tabs(c.get('editor.insert_spaces'))
        self.set_auto_indent(c.get('editor.auto_indent'))
        if c.get('editor.default_font'):
            client = gconf.client_get_default()
            default_font = '/desktop/gnome/interface/monospace_font_name'
            font = client.get_string(default_font)
        else:
            font = c.get('editor.font')
        self.modify_font(pango.FontDescription(font))
        self.set_show_right_margin(c.get('editor.right_margin'))
        self.set_right_margin_position(c.get('editor.right_margin_position'))
        buf.set_highlight_matching_brackets(c.get('editor.bracket_matching'))
        self.set_tab_width(c.get('editor.tabs_width'))
        sm = gtksourceview2.style_scheme_manager_get_default()
        scheme = sm.get_scheme(c.get('editor.scheme'))
        buf.set_style_scheme(scheme)

    def get_statements(self):
        """Finds statements in current buffer.

        Returns:
            List of 2-tuples (start iter, end iter).
        """
        content = self.buffer.get_text(*self.buffer.get_bounds())
        iter = self.buffer.get_start_iter()
        if self.editor and self.editor.connection:
            dialect = self.editor.connection.get_dialect()
        else:
            dialect = None
        for stmt in sqlparse.sqlsplit(content, dialect=dialect):
            if not stmt.strip():
                continue
            start, end = iter.forward_search(stmt.lstrip(),
                                             gtk.TEXT_SEARCH_TEXT_ONLY)
            yield start, end
            iter = end
            if not iter:
                raise StopIteration
        raise StopIteration



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""Application object

"""

import os
import sys

import gobject
import gtk

from cf import release, USER_DIR
from config import Config
from datasources import DatasourceManager
from plugins.core import PluginManager
from ui.prefs import PreferencesDialog
from userdb import UserDB
from dbus_manager import CFService

import logging
log = logging.getLogger("APP")

class CFApplication(gobject.GObject):
    """Main application object

    An instance of this class is accessible in almost every class in
    CrunchyFrog through the ``app`` attribute. It provides access
    to application internals.

    The easiest way to learn more about this object is to activate the
    Python shell plugin. The shell has two local variables: ``app``
    is the CFApplication instance and ``instance`` is the `CFInstance`_
    object for the GUI the shell runs in.

    Attributes
    ==========

        :config: Configuration (see `Config`_ class)
        :userdb: User database (see `UserDB`_ class)
        :plugins: Plugin registry (see `PluginManager`_ class)
        :datasources: Datasource information (see `DatasourceManager`_ class)

    .. _Config: cf.config.Config.html
    .. _UserDB: cf.userdb.UserDB.html
    .. _PluginManager: cf.plugins.core.PluginManager.html
    .. _DatasourceManager: cf.datasources.DatasourceManager.html
    .. _CFInstance: cf.instance.CFInstance.html
    """

    def __init__(self, options):
        """
        The constructor of this class takes one argument:

        :Parameter:
            options
                Commandline options as returned by `optparse`_

        .. _optparse: http://docs.python.org/lib/module-optparse.html
        """
        self.__gobject_init__()
        self.set_data("instances", {})
        self.__icount = 0L
        self.options = options

    def init(self):
        """Initializes the application"""
        self.cb = CFAppCallbacks()
        self.__shutdown_tasks = []
        self._check_version()
        self.config = Config(self, self.options.config)
        self.userdb = UserDB(self)
        self.plugins = PluginManager(self)
        self.datasources = DatasourceManager(self)
        self.dbus_service = CFService(self)
        self.recent_manager = gtk.recent_manager_get_default()

    def _check_version(self):
        """Run eventual version updates."""
        version_file = os.path.join(USER_DIR, 'VERSION')
        if not os.path.isfile(version_file):
            f = open(version_file, "w")
            f.write("0.3.0")
            f.close()
        # That's it for now, it's for future use.

    def new_instance(self, args=None):
        """Creates a new instances.

        ``args`` is an optional list of file names to open
        """
        from cf.instance import CFInstance
        instance = CFInstance(self)
        instances = self.get_data("instances")
        self.__icount += 1
        instances[self.__icount] = instance
        self.set_data("instances", instances)
        instance.set_data("instance-id", self.__icount)
        instance.widget.connect("destroy", self.on_instance_destroyed, self.__icount)
        instance.widget.show()
        self.cb.emit("instance-created", instance)
        return instance

    def on_instance_destroyed(self, instance, instance_id):
        instances = self.get_data("instances")
        if instances.has_key(instance_id):
            del instances[instance_id]
        self.set_data("instances", instances)
        if not instances:
            self.shutdown()

    def get_instance(self, instance_id):
        """Returns an instances identified by ID"""
        return self.get_data("instances").get(instance_id)

    def get_instances(self):
        """Returns a list of active instances

        :Returns: List of `CFInstance`_ instances

        .. _CFInstance: cf.instance.CFInstance.html
        """
        return self.get_data("instances").values()

    def preferences_show(self, mode="editor"):
        """Displays the preferences dialog"""
        dlg = PreferencesDialog(self, mode=mode)
        dlg.run() # IGNORE:E1101 - Generic method
        dlg.destroy() # IGNORE:E1101 - Generic method

    def register_shutdown_task(self, func, msg, *args, **kwargs):
        """Registers a new shutdown task.

        Shutdown tasks are executed when the ``shutdown`` method is
        called.

        :Parameter:
            func
                A callable
            msg
                A human readable message explaingin what the task does
            args, kwargs
                arguments and keyword arguments for ``func`` (optional)
        """
        self.__shutdown_tasks.append((func, msg, args, kwargs))

    def shutdown(self):
        """Execute all shutdown tasks and quit application

        Shutdown tasks are called within a try-except-block. If an
        exception is raised it will be printed to stdout.
        """
        while self.__shutdown_tasks:
            func, msg, args, kwargs = self.__shutdown_tasks.pop()
            log.info(msg)
            try:
                func(*args, **kwargs)
            except:
                import traceback; traceback.print_exc()
                log.error("Task failed: %s" % str(sys.exc_info()[1]))
        gtk.main_quit()


class CFAppCallbacks(gobject.GObject):
    """Container for application specific signals.

    Signals
    =======

        instance-created
            ``def callback(application, instance, user_data1, ...)``

            Called when a new instance was created
    """

    __gsignals__ = {
        "instance-created" : (gobject.SIGNAL_RUN_LAST,
                              gobject.TYPE_NONE,
                              (gobject.TYPE_PYOBJECT,)),
    }



# Copyright (C) 2008 Andi Albrecht, albrecht.andi@gmail.com
#
# This module is part of python-sqlparse and is released under
# the BSD License: http://www.opensource.org/licenses/bsd-license.php.

"""This module contains classes that represent SQL dialects."""

import re

_TOKENIZER_REGEX = r"""%(multiwordkeywords)s|
\s+|
;|
(?:'[^'\n\r]*')+|
<=|>=|!=|=|<|>|
:\w+|
,|
\*|
-?\d+(?:\.\d+)?|
\w+|
\(|\)|
\S+|
"""


class Dialect(object):

    operators = []

    def __init__(self):
        if hasattr(self, 'multi_word_keywords'):
            multiwordkeywords = '|'.join(self.multi_word_keywords)
        else:
            multiwordkeywords = ''
        regex = _TOKENIZER_REGEX % {'multiwordkeywords': multiwordkeywords}
        self._tokenizer_regex = re.compile(regex,
                                           re.VERBOSE | re.IGNORECASE)

    def semicolon_split_level(self, token):
        return 0

    @property
    def tokenizer_regex(self):
        return self._tokenizer_regex


class DialectDefault(Dialect):

    multi_word_keywords = [
        'END\sIF',
        'END\sLOOP',
    ]

    operators = [
        '=', '<', '>', '<=', '>=', 'IN'
    ]

    def __init__(self):
        super(DialectDefault, self).__init__()
        self._in_declare = False
        self._in_create = False

    def semicolon_split_level(self, token):
        """Returns a value to adjust the semicolon split level.

         0: Don't change level.
         1: Increase level.
        -1: Decrease level.

        For example, if token is 'BEGIN' 1 should be returned.
        """
        if token == 'DECLARE':
            self._in_declare = True
            return 1
        if token == 'CREATE':
            self._in_create = True
            return 0
        if token == 'BEGIN':
            if self._in_declare or not self._in_create:
                return 0
            return 1
        if token == 'END':
            return -1
        return 0

    def _get_operators(self):
        return super(DialectDefault, self)._get_operators()+self._operators


class DialectPSQL(DialectDefault):

    operators = DialectDefault.operators + [
        'LIKE', '~'
    ]

    def __init__(self):
        super(DialectPSQL, self).__init__()
        self._in_dbldollar = False

    def semicolon_split_level(self, token):
        if token == '$$':
            if not self._in_dbldollar:
                self._in_dbldollar = True
                return 1
            else:
                self._in_dbldollar = False
                return -1
        else:
            return super(DialectPSQL, self).semicolon_split_level(token)



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""Preferences"""

import gtk
import gobject
import gnome
import gconf
import gnomevfs

import os
import sys

from gettext import gettext as _

import logging
log = logging.getLogger("PREFS")

from inspect import isclass

try:
    from xml.etree import ElementTree as etree
except ImportError:
    from lxml import etree
from kiwi.ui import dialogs

import cf
from cf.ui import GladeWidget
from cf.ui.widgets import ProgressDialog
from cf.plugins.core import GenericPlugin

class PreferencesDialog(GladeWidget):

    def __init__(self, app, mode="editor"):
        GladeWidget.__init__(self, app, "crunchyfrog", "preferences_dialog")
        self.sync_repo_file(silent=True)
        self.refresh()
        if mode == "editor":
            curr_page = 1
        elif mode == "plugins":
            curr_page = 2
        else:
            curr_page = 0
        self.xml.get_widget("notebook1").set_current_page(curr_page)

    def _setup_widget(self):
        self._setup_plugins()
        self._setup_editor()
        # Fix secondary button
        btn = self.xml.get_widget("btn_help")
        box = self.xml.get_widget("dialog-action_area1")
        box.set_child_secondary(btn, True)

    def _setup_editor(self):
        model = gtk.ListStore(int, str, gtk.gdk.Pixbuf)
        iconview = self.xml.get_widget("editor_iconview")
        iconview.set_model(model)
        iconview.set_text_column(1)
        iconview.set_pixbuf_column(2)
        it = gtk.icon_theme_get_default()
        def get_pb(stock):
            return it.load_icon(stock, 36, gtk.ICON_LOOKUP_FORCE_SVG)
        model.append([0, _(u"General"), get_pb("gtk-execute")])
        model.append([1, _(u"View"), get_pb("gtk-justify-left")])
        model.append([2, _(u"Editor"), get_pb("gtk-edit")])
        model.append([3, _(u"Fonts & Colors"), get_pb("gtk-font")])
        iconview.connect("selection-changed", self.on_editor_selection_changed)
        schemes = self.xml.get_widget("editor_schemes")
        model = gtk.ListStore(str, str)
        model.set_sort_column_id(1, gtk.SORT_ASCENDING)
        schemes.set_model(model)
        col = gtk.TreeViewColumn("", gtk.CellRendererText(), markup=1)
        schemes.append_column(col)
        import gtksourceview2
        sm = gtksourceview2.style_scheme_manager_get_default()
        selected = None
        for id in sm.get_scheme_ids():
            scheme = sm.get_scheme(id)
            lbl = "<b>%s</b>" % scheme.get_name()
            if scheme.get_description():
                lbl += "\n"+scheme.get_description()
            iter = model.append(None)
            model.set(iter, 0, id, 1, lbl)
            if id == self.app.config.get("editor.scheme"):
                selected = iter
        sel = schemes.get_selection()
        sel.select_iter(selected)
        sel.connect("changed", self.on_editor_option_changed)
        sel.set_data("config_option", "editor.scheme")


    def _setup_plugins(self):
        """Set up the plugins view"""
        self.plugin_model = gtk.TreeStore(gobject.TYPE_PYOBJECT, # 0 Plugin class
                                          bool, # 1 active
                                          str, # 2 label
                                          gtk.gdk.Pixbuf, # 3 icon
                                          bool, # 4 active visible
                                          )
        self.plugin_model.set_sort_column_id(2, gtk.SORT_ASCENDING)
        self.plugin_list = self.xml.get_widget("plugin_list")
        self.plugin_list.set_model(self.plugin_model)
        # label
        col = gtk.TreeViewColumn()
        renderer = gtk.CellRendererToggle()
        renderer.connect("toggled", self.on_plugin_active_toggled)
        col.pack_start(renderer, expand=False)
        col.add_attribute(renderer, "active", 1)
        col.add_attribute(renderer, "visible", 4)
        renderer = gtk.CellRendererPixbuf()
        col.pack_start(renderer, expand=False)
        col.add_attribute(renderer, 'pixbuf', 3)
        renderer = gtk.CellRendererText()
        col.pack_start(renderer, expand=True)
        col.add_attribute(renderer, 'markup', 2)
        self.plugin_list.append_column(col)
        sel = self.plugin_list.get_selection()
        sel.connect("changed", self.on_plugin_selection_changed)

    def _setup_connections(self):
        self.app.plugins.connect("plugin-added", self.on_plugin_added)
        self.app.plugins.connect("plugin-removed", self.on_plugin_removed)

    def on_editor_open_in_window_toggled(self, toggle):
        self.app.config.set("editor.open_in_window", toggle.get_active())

    def on_editor_option_changed(self, widget, *args):
        option = widget.get_data("config_option")
        conf = self.app.config
        if isinstance(widget, gtk.CheckButton):
            conf.set(option, widget.get_active())
        elif isinstance(widget, gtk.SpinButton):
            conf.set(option, widget.get_value_as_int())
        elif isinstance(widget, gtk.FontButton):
            conf.set(option, widget.get_font_name())
        elif isinstance(widget, gtk.TreeSelection):
            model, iter = widget.get_selected()
            conf.set(option, model.get_value(iter, 0))
        if option == "editor.wrap_text":
            self.xml.get_widget("editor_wrap_split").set_sensitive(widget.get_active())
        if option == "editor.right_margin":
            self.xml.get_widget("editor_right_margin_position_box").set_sensitive(widget.get_active())
        if option == "editor.default_font":
            self.xml.get_widget("editor_font_box").set_sensitive(not widget.get_active())
        if option == "plugins.repo_enabled":
            gobject.idle_add(self.refresh_plugins)

    def on_editor_selection_changed(self, iconview):
        model = iconview.get_model()
        for path in iconview.get_selected_items():
            iter = model.get_iter(path)
            nb = self.xml.get_widget("editor_notebook")
            nb.set_current_page(model.get_value(iter, 0))

    def on_plugin_active_toggled(self, renderer, path):
        iter = self.plugin_model.get_iter(path)
        plugin = self.plugin_model.get_value(iter, 0)
        if isinstance(plugin, etree._Element):
            self._plugin_download_and_activate(plugin)
        elif issubclass(plugin, GenericPlugin):
            self.plugin_model.set_value(iter, 1, not renderer.get_active())
            gobject.idle_add(self.app.plugins.set_active, plugin, not renderer.get_active())

    def _plugin_download_and_activate(self, plugin):
        if not dialogs.yesno(_(u"Download plugin %r?") % plugin.xpath("//*/name")[0].text) == gtk.RESPONSE_YES:
            return
        def progress_cb(info, dlg):
            if info.bytes_total:
                fraction = info.total_bytes_copied/float(info.bytes_total)
            else:
                fraction = 0
            dlg.set_progress(fraction)
            return True
        url = os.path.join(self.app.config.get("plugins.repo_url"), plugin.xpath("//*/egg")[0].text)
        source = gnomevfs.URI(url)
        dest = gnomevfs.URI(os.path.join(cf.USER_PLUGIN_URI, plugin.xpath("//*/egg")[0].text))
        try:
            dlg = ProgressDialog(self.app)
            dlg.show_all()
            dlg.set_info(_(u"Downloading plugin %r...") % plugin.xpath("//*/name")[0].text)
            gnomevfs.xfer_uri(source, dest, gnomevfs.XFER_DEFAULT,
                              gnomevfs.XFER_ERROR_MODE_QUERY,
                              gnomevfs.XFER_OVERWRITE_MODE_QUERY,
                              progress_cb, dlg)
            dlg.set_info(_(u"Plugin downloaded."))
            self.app.plugins.refresh()
            self.refresh_plugins_repo()
        except:
            err = sys.exc_info()[1]
            log.error("Plugin download failed: %s" % err)
            log.error("URL: %s" % url)
            dlg.set_error(_(u"Failed to install plugin %r") % plugin.xpath("//*/name")[0].text)
        dlg.set_finished(True)

    def on_plugin_added(self, manager, plugin):
        iter = self.plugin_model.get_iter_first()
        it = gtk.icon_theme_get_default()
        while iter:
            if self.plugin_model.get_value(iter, 0) == plugin.plugin_type:
                break
            iter = self.plugin_model.iter_next(iter)
        if iter:
            lbl = '<b>%s</b>' % plugin.name or _(u"Unknown")
            if plugin.description:
                lbl += "\n"+plugin.description
            if plugin.icon:
                ico = it.load_icon(plugin.icon, gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_FORCE_SVG)
            else:
                ico = None
            citer = self.plugin_model.append(iter)
            self.plugin_model.set(citer,
                      0, plugin,
                      1, self.app.plugins.is_active(plugin),
                      2, lbl,
                      3, ico,
                      4, True)
        self.refresh_plugins_repo()

    def on_plugin_install(self, *args):
        dlg = gtk.FileChooserDialog(_(u"Install plugin"), None,
                                    gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        filter = gtk.FileFilter()
        filter.set_name(_(u"CrunchyFrog plugins (*.zip, *.py)"))
        filter.add_pattern("*.zip")
        filter.add_pattern("*.py")
        dlg.add_filter(filter)
        dlg.set_filter(filter)
        if dlg.run() == gtk.RESPONSE_OK:
            uri = dlg.get_uri()
        else:
            uri = None
        dlg.destroy()
        gobject.idle_add(self.app.plugins.install_plugin, uri)

    def on_plugin_removed(self, manager, plugin):
        iter = self.plugin_model.get_iter_first()
        while iter:
            if self.plugin_model.get_value(iter, 0) == plugin.plugin_type:
                citer = self.plugin_model.iter_children(iter)
                while citer:
                    if self.plugin_model.get_value(citer, 0) == plugin:
                        self.plugin_model.remove(citer)
                        self.refresh_plugins_repo()
                        return
                    citer = self.plugin_model.iter_next(citer)
            iter = self.plugin_model.iter_next(iter)

    def on_plugin_prefs_show(self, *args):
        sel = self.plugin_list.get_selection()
        model, iter = sel.get_selected()
        if not iter: return
        obj = model.get_value(iter, 0)
        if not isclass(obj) or not issubclass(obj, GenericPlugin):
            return
        if not obj.has_custom_options:
            return
        obj.run_custom_options_dialog(self.app)

    def on_plugin_show_about(self, *args):
        sel = self.plugin_list.get_selection()
        model, iter = sel.get_selected()
        if not iter: return
        obj = model.get_value(iter, 0)
        if not isclass(obj) or not issubclass(obj, GenericPlugin):
            return
        dlg = gtk.AboutDialog()
        if obj.name: dlg.set_name(obj.name)
        if obj.description: dlg.set_comments(obj.description)
        if obj.icon: dlg.set_logo_icon_name(obj.icon)
        if obj.author: dlg.set_authors([obj.author])
        if obj.license: dlg.set_license(obj.license)
        if obj.homepage: dlg.set_website(obj.homepage)
        if obj.version: dlg.set_version(obj.version)
        dlg.run()
        dlg.destroy()

    def on_plugin_selection_changed(self, selection, *args):
        model, iter = selection.get_selected()
        if not iter:
            return
        obj = model.get_value(iter, 0)
        if isclass(obj) and issubclass(obj, GenericPlugin):
            self.xml.get_widget("plugin_about").set_sensitive(True)
            self.xml.get_widget("plugin_prefs").set_sensitive(obj.has_custom_options)
        else:
            self.xml.get_widget("plugin_about").set_sensitive(False)
            self.xml.get_widget("plugin_prefs").set_sensitive(False)

    def on_plugin_sync_repo(self, *args):
        self.sync_repo_file()

    def on_plugin_folder_show(self, *args):
        gnome.url_show(cf.USER_PLUGIN_URI)

    def refresh(self):
        self.refresh_editor()
        self.refresh_plugins()

    def refresh_editor(self):
        config = self.app.config
        gw = self.xml.get_widget
        gw("editor_open_separate").set_active(config.get("editor.open_in_window"))
        gw("editor_replace_variables").set_data("config_option", "editor.replace_variables")
        gw("editor_replace_variables").set_active(config.get("editor.replace_variables"))
        gw("sqlparse_enabled").set_data("config_option", "sqlparse.enabled")
        gw("sqlparse_enabled").set_active(config.get("sqlparse.enabled"))
        gw("editor_wrap_text").set_data("config_option", "editor.wrap_text")
        gw("editor_wrap_text").set_active(config.get("editor.wrap_text"))
        gw("editor_wrap_split").set_data("config_option", "editor.wrap_split")
        gw("editor_wrap_split").set_active(config.get("editor.wrap_split"))
        gw("editor_wrap_split").set_sensitive(gw("editor_wrap_text").get_active())
        gw("editor_display_line_numbers").set_data("config_option", "editor.display_line_numbers")
        gw("editor_display_line_numbers").set_active(config.get("editor.display_line_numbers"))
        gw("editor_highlight_current_line").set_data("config_option", "editor.highlight_current_line")
        gw("editor_highlight_current_line").set_active(config.get("editor.highlight_current_line"))
        gw("editor_right_margin").set_data("config_option", "editor.right_margin")
        gw("editor_right_margin").set_active(config.get("editor.right_margin"))
        gw("editor_right_margin_position").set_data("config_option", "editor.right_margin_position")
        gw("editor_right_margin_position").set_value(config.get("editor.right_margin_position"))
        gw("editor_right_margin_position_box").set_sensitive(config.get("editor.right_margin"))
        gw("editor_bracket_matching").set_data("config_option", "editor.bracket_matching")
        gw("editor_bracket_matching").set_active(config.get("editor.bracket_matching"))
        gw("editor_tabs_width").set_data("config_option", "editor.tabs_width")
        gw("editor_tabs_width").set_value(config.get("editor.tabs_width"))
        gw("editor_insert_spaces").set_data("config_option", "editor.insert_spaces")
        gw("editor_insert_spaces").set_active(config.get("editor.insert_spaces"))
        gw("editor_auto_indent").set_data("config_option", "editor.auto_indent")
        gw("editor_auto_indent").set_active(config.get("editor.auto_indent"))
        gw("editor_default_font").set_data("config_option", "editor.default_font")
        gw("editor_default_font").set_active(config.get("editor.default_font"))
        client = gconf.client_get_default()
        default_font = client.get_string("/desktop/gnome/interface/monospace_font_name")
        gw("editor_default_font").set_label(gw("editor_default_font").get_label() % default_font)
        gw("editor_font_box").set_sensitive(not config.get("editor.default_font"))
        gw("editor_font").set_data("config_option", "editor.font")
        gw("editor_font").set_font_name(config.get("editor.font"))

    def refresh_plugins(self):
        # Repo
        self.xml.get_widget("plugin_enable_repo").set_data("config_option", "plugins.repo_enabled")
        self.xml.get_widget("plugin_enable_repo").set_active(self.app.config.get("plugins.repo_enabled"))
        # Plugins
        model = self.plugin_model
        model.clear()
        it = gtk.icon_theme_get_default()
        for key, value in self.app.plugins.plugin_types.items():
            iter = model.append(None)
            model.set(iter,
                      0, key,
                      1, False,
                      2, '<b>%s</b>' % value[0],
                      3, None,
                      4, False)
            for plugin in self.app.plugins.get_plugins(key):
                lbl = '<b>%s</b>' % plugin.name or _(u"Unknown")
                if plugin.description:
                    lbl += "\n"+plugin.description
                if plugin.INIT_ERROR:
                    lbl += '\n<span color="red">ERROR: '+plugin.INIT_ERROR+'</span>'
                if plugin.icon:
                    ico = it.load_icon(plugin.icon, gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_FORCE_SVG)
                else:
                    ico = None
                citer = model.append(iter)
                model.set(citer,
                          0, plugin,
                          1, self.app.plugins.is_active(plugin),
                          2, lbl,
                          3, ico,
                          4, not bool(plugin.INIT_ERROR))
        gobject.idle_add(self.refresh_plugins_repo)

    def _plugin_iter_for_ep(self, ep_name):
        model = self.plugin_list.get_model()
        iter = model.get_iter_first()
        while iter:
            if model.get_value(iter, 0) == ep_name:
                return model, iter
            iter = model.iter_next(iter)
        return model, None

    def refresh_plugins_repo(self):
        """Refresh repository plugins"""
        model = self.plugin_list.get_model()
        iter = model.get_iter_first()
        while iter:
            citer = model.iter_children(iter)
            while citer:
                obj = model.get_value(citer, 0)
                if isinstance(obj, etree._Element):
                    model.remove(citer)
                citer = model.iter_next(citer)
            iter = model.iter_next(iter)
        if not self.app.config.get("plugins.repo_enabled"):
            return
        if not os.path.isfile(cf.USER_PLUGIN_REPO):
            if not self.sync_repo_file():
                return
        dom = etree.parse(cf.USER_PLUGIN_REPO)
        for plugin in dom.xpath("//*/plugin"):
            ptype = plugin.get("type")
            model, iter = self._plugin_iter_for_ep(int(ptype))
            if not iter:
                log.error("Invalid plugin type %r" % ptype)
                continue
            citer = model.iter_children(iter)
            skip = False
            while citer:
                x = model.get_value(citer, 0)
                if isclass(x) and issubclass(x, GenericPlugin) \
                and x.id == plugin.get("id"):
                    skip = True
                    break
                elif isinstance(x, etree._Element) \
                and x.get("id") == plugin.get("id"):
                    skip = True
                    break
                citer = model.iter_next(citer)
            if skip:
                continue
            lbl = '<b>[REPO] %s</b>' % plugin.xpath("//*/name")[0].text or _(u"Unknown")
            if plugin.xpath("//*/description"):
                lbl += "\n"+plugin.xpath("//*/description")[0].text
            it = gtk.icon_theme_get_default()
            ico = it.load_icon("stock_internet", gtk.ICON_SIZE_LARGE_TOOLBAR, gtk.ICON_LOOKUP_FORCE_SVG)
            citer = model.append(iter)
            model.set(citer,
                      0, plugin,
                      1, False,
                      2, lbl,
                      3, ico,
                      4, True)

    def sync_repo_file(self, silent=False):
        """Synchronize plugin repo.xml"""
        def progress_cb(info, dlg):
            if info.bytes_total:
                fraction = info.total_bytes_copied/float(info.bytes_total)
            else:
                fraction = 0
            dlg.set_progress(fraction)
            return True
        # XXX currently disabled, should be replaced by a Capuchin backend
        return
        url = os.path.join(self.app.config.get("plugins.repo_url"), "repo.xml")
        source = gnomevfs.URI(url)
        dest = gnomevfs.URI(cf.USER_PLUGIN_REPO_URI)
        try:
            if not silent:
                dlg = ProgressDialog(self.app)
                dlg.show_all()
                dlg.set_info(_(u"Downloading repository data..."))
                gnomevfs.xfer_uri(source, dest, gnomevfs.XFER_DEFAULT,
                                  gnomevfs.XFER_ERROR_MODE_QUERY,
                                  gnomevfs.XFER_OVERWRITE_MODE_QUERY,
                                  progress_cb, dlg)
                dlg.set_info(_(u"Repository data synchronized."))
                self.refresh_plugins_repo()
            else:
                def fake_cb(*args):
                    return True
                gnomevfs.xfer_uri(source, dest, gnomevfs.XFER_DEFAULT,
                                  gnomevfs.XFER_ERROR_MODE_QUERY,
                                  gnomevfs.XFER_OVERWRITE_MODE_QUERY,
                                  fake_cb)
            retval = True
        except:
            err = sys.exc_info()[1]
            log.error("Plugin synchro failed: %s" % err)
            log.error("URL: %s" % url)
            if not silent:
                dlg.set_error(_(u"Failed to load repository data"))
            self.xml.get_widget("plugin_enable_repo").set_active(False)
            retval = False
        if not silent:
            dlg.set_finished(True)
        return retval



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""PostgreSQL backend"""

import gtk
import gobject

from cf.backends import DBConnectError, TRANSACTION_IDLE, TRANSACTION_COMMIT_ENABLED, TRANSACTION_ROLLBACK_ENABLED
from cf.backends.dbapi2helper import DbAPI2Connection, DbAPI2Cursor
from cf.backends.schema import *
from cf.backends import ReferenceProvider
from cf.datasources import DatasourceInfo
from cf.plugins.core import DBBackendPlugin
from cf import sqlparse
from cf.utils import Emit

import time
from urllib import quote_plus


from gettext import gettext as _

import logging
log = logging.getLogger("PG")

class PgReferenceProvider(ReferenceProvider):
    name = _(u"PostgreSQL Reference")
    base_url = "http://www.postgresql.org/docs/current"

    def get_context_help_url(self, term):
        url = "http://search.postgresql.org/search?u=%2Fdocs%2F8.3%2Finteractive%2F&q="
        url += quote_plus(term.strip())
        return url

class PostgresBackend(DBBackendPlugin):

    id = "crunchyfrog.backend.postgres"
    name = _(u"PostgreSQL Plugin")
    description = _(u"Provides access to PostgreSQL databases")
    password_option = "password"

    def __init__(self, app):
        DBBackendPlugin.__init__(self, app)
        log.info("Activating PostgreSQL backend")
        self.schema = PgSchema()
        self.reference = PgReferenceProvider()

    def _get_conn_opts_from_opts(self, opts):
        conn_opts = dict()
        if opts["database"]: conn_opts["database"] = opts["database"]
        if opts["host"]: conn_opts["host"] = opts["host"]
        if opts["port"]: conn_opts["port"] = opts["port"]
        if opts["user"]: conn_opts["user"] = opts["user"]
        if opts["password"]: conn_opts["password"] = opts["password"]
        return conn_opts

    def shutdown(self):
        log.info("Shutting down PostgreSQL backend")

    @classmethod
    def get_datasource_options_widgets(cls, data_widgets, initial_data=None):
        return data_widgets, ["database", "host", "port", "user", "password"]

    @classmethod
    def get_label(cls, datasource_info):
        if not datasource_info.options.get("host", None):
            datasource_info.options["host"] = "localhost"
        return DBBackendPlugin.get_label(datasource_info)

    def dbconnect(self, data):
        opts = self._get_conn_opts_from_opts(data)
        if data.get("ask_for_password", False):
            pwd = self.password_prompt()
            if not pwd:
                raise DBConnectError(_(u"No password given."))
            opts["password"] = pwd
        try:
            import psycopg2
        except ImportError:
            raise DBConnectError(_(u"Python module psycopg2 is not installed."))
        try:
            real_conn = psycopg2.connect(**opts)
            real_conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
        except StandardError, err:
            raise DBConnectError(str(err))
        conn = PgConnection(self, self.app, real_conn)
        conn.threadsafety = psycopg2.threadsafety
        return conn

    def test_connection(self, data):
        try:
            conn = self.dbconnect(data)
            conn.close()
        except DBConnectError, err:
            return str(err)
        return None

class PgCursor(DbAPI2Cursor):

    def __init__(self, *args, **kw):
        DbAPI2Cursor.__init__(self, *args, **kw)
        self._notice_tag = None

    def _check_notices(self):
        return
        while self.connection._conn.notices:
            gobject.idle_add(self.connection.emit, "notice", self.connection._conn.notices.pop())
            #Emit(self.connection, "notice", self.connection._conn.notices.pop())

    def execute(self, *args, **kw):
        #gtk.gdk.threads_enter()
        #self._notice_tag = gobject.timeout_add(100, self._check_notices)
        #gtk.gdk.threads_leave()
        DbAPI2Cursor.execute(self, *args, **kw)
        #gtk.gdk.threads_enter()
        #gobject.source_remove(self._notice_tag)
        #gtk.gdk.threads_leave()
        #self._check_notices()

    def get_messages(self):
        if self._cur.statusmessage:
            return [self._cur.statusmessage]
        else:
            return []

class PgConnection(DbAPI2Connection):
    cursor_class = PgCursor
    sqlparse_dialect = sqlparse.DialectPSQL()

    def update_transaction_status(self):
        stat = self._conn.get_transaction_status()
        if stat == psycopg2.extensions.TRANSACTION_STATUS_INERROR:
            flag = TRANSACTION_ROLLBACK_ENABLED
        elif stat == psycopg2.extensions.TRANSACTION_STATUS_ACTIVE:
            flag = TRANSACTION_IDLE
        elif stat == psycopg2.extensions.TRANSACTION_STATUS_IDLE:
            flag = TRANSACTION_IDLE
        elif stat == psycopg2.extensions.TRANSACTION_STATUS_INTRANS:
            flag = TRANSACTION_COMMIT_ENABLED|TRANSACTION_ROLLBACK_ENABLED
        else:
            flag = TRANSACTION_IDLE
        self.props.transaction_state = flag

    def get_server_info(self):
        cur = self._conn.cursor()
        cur.execute("select version()")
        ret = cur.fetchone()
        cur.close()
        return ret[0]

class PgLanguageCollection(Collection):
    name = _(u"Languages")

class PgLanguage(Node):
    name = _(u"Language")
    icon = "stock_script"
    has_children = False

class PgSchema(SchemaProvider):

    def __init__(self):
        SchemaProvider.__init__(self)

    def q(self, connection, sql):
        cur = connection.cursor()._cur
        cur.execute(sql)
        return cur.fetchall()

    def fetch_children(self, connection, parent):
        if isinstance(parent, DatasourceInfo):
            return [SchemaCollection(),
                    PgLanguageCollection()]

        elif isinstance(parent, SchemaCollection):
            ret = []
            sql = "select nsp.oid, nsp.nspname, dsc.description \
            from pg_namespace nsp \
            left join pg_description dsc on dsc.objoid = nsp.oid \
            where nsp.nspname not like 'pg_%' and nsp.nspname != 'information_schema'"
            for item in self.q(connection, sql):
                s = Schema(item[1], item[2])
                s.set_data("oid", item[0])
                ret.append(s)
            return ret

        elif isinstance(parent, Schema):
            return [TableCollection(schema=parent),
                    ViewCollection(schema=parent),
                    SequenceCollection(schema=parent),
                    FunctionCollection(schema=parent)]

        elif isinstance(parent, TableCollection) \
        or isinstance(parent, ViewCollection) \
        or isinstance(parent, SequenceCollection):
            if isinstance(parent, TableCollection):
                obj = Table
                relkind = 'r'
                has_details = False
            elif isinstance(parent, ViewCollection):
                obj = View
                relkind = 'v'
                has_details = True
            elif isinstance(parent, SequenceCollection):
                obj = Sequence
                relkind = 'S'
                has_details = False
            schema = parent.get_data("schema")
            sql = "select rel.oid, rel.relname, dsc.description from pg_class rel \
            left join pg_description dsc on dsc.objoid = rel.oid and dsc.objsubid = 0 \
            where rel.relnamespace = %(nspoid)s \
            and rel.relkind = '%(relkind)s'" % {"nspoid" : schema.get_data("oid"),
                                                "relkind" : relkind}
            ret = []
            for item in self.q(connection, sql):
                ret.append(obj(item[1], item[2], oid=item[0],
                               has_details=has_details))
            return ret

        elif isinstance(parent, Table):
            return [ColumnCollection(table=parent),
                    ConstraintCollection(table=parent),
                    IndexCollection(table=parent)]

        elif isinstance(parent, View):
            return [ColumnCollection(table=parent)]

        elif isinstance(parent, ColumnCollection):
            table = parent.get_data("table")
            ret = []
            sql = "select att.attnum, att.attname, dsc.description from pg_attribute att \
            left join pg_description dsc on dsc.objoid = %(tableoid)s and dsc.objsubid = att.attnum \
            where att.attrelid = %(tableoid)s \
            and att.attnum >= 1" % {"tableoid" : table.get_data("oid")}
            for item in self.q(connection, sql):
                ret.append(Column(item[1], item[2], attnum=item[0]))
            return ret

        elif isinstance(parent, FunctionCollection):
            ret = []
            schema = parent.get_data("schema")
            sql = "select pro.oid, pro.proname, dsc.description from pg_proc pro \
            left join pg_description dsc on dsc.objoid = pro.oid \
            where pro.pronamespace = %(schemaoid)s" % {"schemaoid" : schema.get_data("oid")}
            for item in self.q(connection, sql):
                ret.append(Function(item[1], item[2], oid=item[0]))
            return ret

        elif isinstance(parent, ConstraintCollection):
            ret = []
            table = parent.get_data("table")
            sql = "select con.oid, con.conname \
            from pg_constraint con \
            where con.conrelid = %(relid)s" % {"relid" : table.get_data("oid")}
            for item in self.q(connection, sql):
                ret.append(Constraint(item[1], oid=item[0]))
            return ret

        elif isinstance(parent, IndexCollection):
            ret = []
            table = parent.get_data("table")
            sql = "select rel.oid, rel.relname, dsc.description \
            from pg_class rel \
            left join pg_description dsc on dsc.objoid = rel.oid \
            , pg_index ind \
            where ind.indrelid = %(relid)s and ind.indexrelid = rel.oid" % {"relid" : table.get_data("oid")}
            for item in self.q(connection, sql):
                ret.append(Index(item[1], item[2], oid=item[0]))
            return ret

        elif isinstance(parent, PgLanguageCollection):
            sql = "select lan.oid, lan.lanname, dsc.description \
            from pg_language lan \
            left join pg_description dsc on dsc.objoid = lan.oid"
            return [PgLanguage(item[1], item[2], oid=item[0]) for item in self.q(connection, sql)]

    def get_details(self, connection, obj):
        func = getattr(self, "details_%s" % obj.__class__.__name__.lower(), None)
        if func:
            return func(connection, obj)
        return None

    def details_view(self, connection, view):
        ret = dict()
        sql = "select pg_get_viewdef(%(oid)s, true)" %{"oid" : view.get_data("oid")}
        res = self.q(connection, sql)
        ret[_(u"Definition")] =  res[0][0]
        return ret


try:
    import psycopg2
    import psycopg2.extensions
except ImportError:
    PostgresBackend.INIT_ERROR = _(u"Python module psycopg2 required.")



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""SQL library

This module contains the SQL library plugin.

:ToDo's:
    - Management interface (reorder, delete, rename, edit, export, dump, ...)
"""

import gtk
import gobject

from kiwi.ui import dialogs

from cf.plugins.core import GenericPlugin
from cf.plugins.mixins import MenubarMixin, EditorMixin, UserDBMixin
from cf.ui import GladeWidget
from cf.ui.widgets import CustomImageMenuItem
from cf.ui.widgets.sqlview import SQLView

from gettext import gettext as _

class SQLLibraryPlugin(GenericPlugin, MenubarMixin, EditorMixin,
                       UserDBMixin):
    """SQL library plugin"""
    id = "crunchyfrog.plugin.library"
    name = _(u"Library")
    description = _(u"A personal SQL library")
    icon = "stock_book_green"
    author = "Andi Albrecht"
    license = "GPL"
    homepage = "http://cf.andialbrecht.de"
    version = "0.1"

    def __init__(self, app):
        GenericPlugin.__init__(self, app)
        self._mn_add = dict()
        self._menu_items = dict()

    def on_add_to_library(self, menuitem, instance):
        editor = instance.get_editor()
        if not editor:
            return
        buffer = editor.textview.get_buffer()
        txt = buffer.get_text(*buffer.get_bounds())
        self.add_to_library(txt)

    def on_load_from_library(self, menuitem, id_, instance):
        editor = instance.get_editor()
        if not editor:
            editor = instance.new_editor()
        sql = "select statement from sqllib_sql where id = %s" % id_
        self.userdb.cursor.execute(sql)
        editor.textview.get_buffer().set_text(self.userdb.cursor.fetchone()[0])

    def on_show_library(self, *args):
        self.show_library()

    def add_to_library(self, txt):
        dlg = LibSaveDialog(self.app)
        dlg.set_statement(txt)
        if dlg.run():
            name = dlg.xml.get_widget("libsave_name").get_text().strip()
            description = dlg.xml.get_widget("libsave_description").get_text().strip() or None
            cat = dlg.get_category()
            sql = "insert into sqllib_sql (title, description, statement, category) values (?, ?, ?, ?);"
            self.userdb.cursor.execute(sql, (name, description, txt, cat))
            self.userdb.conn.commit()
            gobject.idle_add(self.rebuild_menues)
        dlg.destroy()

    def menubar_load(self, menubar, instance):
        children = menubar.get_children()
        pos = -1
        for i in range(len(children)):
            if children[i].get_name() == "mn_help":
                pos = i
                break
        item = gtk.MenuItem(_(u"_Library"))
        item.set_name("library")
        item.show()
        menubar.insert(item, pos)
        menu = gtk.Menu()
        item.set_submenu(menu)
        item = gtk.SeparatorMenuItem()
        item.show()
        menu.append(item)
        self._menu_items[instance] = menu
        item = CustomImageMenuItem("stock_book_open", _(u"_Open library"))
        item.connect("activate", self.on_show_library)
        item.show()
        menu.append(item)
        item = CustomImageMenuItem("gtk-add", _(u"_Add to library"))
        item.set_sensitive(False)
        item.show()
        menu.append(item)
        item.connect("activate", self.on_add_to_library, instance)
        self._mn_add[instance] = item
        self.rebuild_menues()

    def menubar_unload(self, menubar, instance):
        for item in menubar.get_children():
            if item.get_name() == "library":
                menubar.remove(item)
                return
        del self._menu_items[instance]
        del self._mn_add[instance]

    def set_editor(self, editor, instance):
        if not self._mn_add.has_key(instance):
            return
        self._mn_add[instance].set_sensitive(bool(editor))

    def userdb_init(self):
        if self.userdb.get_table_version("sqllib_cat") == None:
            self.userdb.create_table("sqllib_cat", "0.1", TABLE_LIB_CAT)
        if self.userdb.get_table_version("sqllib_sql") == None:
            self.userdb.create_table("sqllib_sql", "0.1", TABLE_LIB_SQL)

    def rebuild_menues(self):
        def add_items(mitem, parent, instance):
            sql = "select id, name from sqllib_cat where "
            if parent:
                sql += "parent = %s" % parent
            else:
                sql += "parent is null"
            sql += " order by name"
            self.userdb.cursor.execute(sql)
            pos = 0
            for entry in self.userdb.cursor.fetchall():
                item = gtk.ImageMenuItem(gtk.STOCK_OPEN)
                item.get_children()[0].set_label(entry[1])
                item.show()
                smenu = gtk.Menu()
                item.set_submenu(smenu)
                add_items(smenu, entry[0], instance)
                mitem.insert(item, pos)
                pos += 1
            sql = "select id, title from sqllib_sql where "
            if parent:
                sql += "category = %s" % parent
            else:
                sql += "category is null"
            self.userdb.cursor.execute(sql)
            for entry in self.userdb.cursor.fetchall():
                item = gtk.ImageMenuItem(gtk.STOCK_JUSTIFY_LEFT)
                item.get_children()[0].set_label(entry[1])
                item.connect("activate", self.on_load_from_library, entry[0], instance)
                item.show()
                mitem.insert(item, pos)
                pos += 1
        for instance, menuitem in self._menu_items.items():
            while menuitem.get_children():
                child = menuitem.get_children()[0]
                if isinstance(child, gtk.SeparatorMenuItem):
                    break
                menuitem.remove(child)
            add_items(menuitem, None, instance)

    def show_library(self):
        dlg = SQLLibraryDialog(self.app, self)
        dlg.run()
        dlg.destroy()

class LibSaveDialog(GladeWidget):
    """Save statement to library dialog"""

    def __init__(self, app):
        GladeWidget.__init__(self, app, "crunchyfrog", "sqllib_save")
        self.init_categories()

    def _setup_widget(self):
        model = gtk.TreeStore(int, str)
        treeview = self.xml.get_widget("libsave_category")
        treeview.set_model(model)
        self.cat_model = model
        renderer = gtk.CellRendererText()
        renderer.connect("edited", self.on_cat_edited)
        renderer.set_property("editable", True)
        col = gtk.TreeViewColumn("", renderer, text=1)
        treeview.append_column(col)
        treeview.connect("button-press-event", self.on_button_press_event)

    def on_add_category(self, menuitem, parent):
        iter = self.cat_model.append(parent)
        sql = "insert into sqllib_cat (name, parent) values (?, ?)"
        if parent:
            parent_id = self.cat_model.get_value(parent, 0)
        else:
            parent_id = None
        name = _(u"New category")
        self.app.userdb.cursor.execute(sql, (name, parent_id))
        id = self.app.userdb.cursor.lastrowid
        self.app.userdb.connection.commit()
        self.cat_model.set(iter, 0, id, 1, name)

    def on_button_press_event(self, treeview, event):
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            popup = gtk.Menu()
            pthinfo = treeview.get_path_at_pos(x, y)
            parent = None
            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                treeview.grab_focus()
                treeview.set_cursor( path, col, 0)
                model = treeview.get_model()
                parent = model.iter_parent(model.get_iter(path))
            item = gtk.MenuItem(_(u"Add category"))
            item.show()
            item.connect("activate", self.on_add_category, parent)
            popup.append(item)
            popup.show_all()
            popup.popup( None, None, None, event.button, time)
            return 1

    def on_cat_edited(self, renderer, path, value):
        iter = self.cat_model.get_iter(path)
        self.cat_model.set_value(iter, 1, value)
        sql = "update sqllib_cat set name = ? where id = ?"
        self.app.userdb.cursor.execute(sql, (value, self.cat_model.get_value(iter, 0)))
        self.app.userdb.connection.commit()

    def on_name_changed(self, *args):
        txt = self.xml.get_widget("libsave_name").get_text().strip()
        self.xml.get_widget("libsave_btn_save").set_sensitive(bool(txt))

    def init_categories(self, parent=None):
        sql = "select id, name from sqllib_cat"
        if parent:
            parent_id = self.cat_model.get_value(parent, 0)
            sql += " where parent = %s" % parent_id
        else:
            sql += " where parent is null"
        sql += " order by name"
        self.app.userdb.cursor.execute(sql)
        for item in self.app.userdb.cursor.fetchall():
            iter = self.cat_model.append(parent)
            self.cat_model.set(iter, 0, item[0], 1, item[1])
            sql = "select count(*) from sqllib_cat where parent = %s" % item[0]
            self.app.userdb.cursor.execute(sql)
            if self.app.userdb.cursor.fetchone()[0]:
                self.init_categories(iter)

    def set_statement(self, statement):
        buffer = self.xml.get_widget("libsave_statement").get_buffer()
        buffer.set_text(statement)

    def get_category(self):
        sel = self.xml.get_widget("libsave_category").get_selection()
        model, iter = sel.get_selected()
        if iter:
            return model.get_value(iter, 0)
        else:
            return None

class SQLLibraryDialog(GladeWidget):

    def __init__(self, app, plugin):
        GladeWidget.__init__(self, app, "crunchyfrog", "sqllib_manage")
        self.plugin = plugin
        self.refresh()

    def _setup_widget(self):
        # treeview
        self.list = self.xml.get_widget("sqlmanage_list")
        self.list.set_reorderable(True)
        model = gtk.TreeStore(gobject.TYPE_PYOBJECT,
                              str,
                              str,
                              bool,
                              str)
        model.set_sort_column_id(4, gtk.SORT_ASCENDING)
        self.list.set_model(model)
        col = gtk.TreeViewColumn()
        renderer = gtk.CellRendererPixbuf()
        col.pack_start(renderer, expand=False)
        col.add_attribute(renderer, 'stock-id', 2)
        renderer = gtk.CellRendererText()
        col.pack_start(renderer, expand=True)
        col.add_attribute(renderer, 'markup', 1)
        self.list.append_column(col)
        # Statement
        self.statement = SQLView(self.app)
        self.xml.get_widget("sqlmanage_sw_statement").add(self.statement)
        self.statement.show_all()

    def _setup_connections(self):
        sel = self.list.get_selection()
        sel.connect("changed", self.on_selection_changed)

    def on_button_pressed(self, treeview, event):
        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            popup = gtk.Menu()
            pthinfo = treeview.get_path_at_pos(x, y)
            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                model = treeview.get_model()
                iter = model.get_iter(path)
                if not model.get_value(iter, 3):
                    item = gtk.MenuItem(_(u"Add category"))
                    item.connect("activate", self.on_category_new, model.get_value(iter, 0))
                    popup.append(item)
                    item = gtk.MenuItem(_(u"Add query"))
                    item.connect("activate", self.on_statement_new, model.get_value(iter,0))
                    popup.append(item)
            else:
                item = gtk.MenuItem(_(u"Add category"))
                item.connect("activate", self.on_category_new, None)
                popup.append(item)
                item = gtk.MenuItem(_(u"Add query"))
                item.connect("activate", self.on_statement_new, None)
                popup.append(item)
            if popup.get_children():
                popup.show_all()
                popup.popup( None, None, None, event.button, time)

    def on_category_new(self, menuitem, parent):
        sql = "insert into sqllib_cat (name, parent) values (?, ?)"
        self.app.userdb.cursor.execute(sql, (_(u"New category"), parent))
        self.app.userdb.connection.commit()
        self.plugin.rebuild_menues()
        self.refresh()

    def on_delete(self, *args):
        selection = self.list.get_selection()
        model, iter = selection.get_selected()
        if not iter: return
        if model.get_value(iter, 3):
            id_ = model.get_value(iter, 0)
            sql = "delete from sqllib_sql where id = %s" % id_
            self.app.userdb.cursor.execute(sql)
            self.app.userdb.connection.commit()
            self.refresh()
            self.plugin.rebuild_menues()
        else:
            id_ = model.get_value(iter, 0)
            sql = "select id from sqllib_cat where parent = %s \
            union select id from sqllib_sql where category = %s"
            sql = sql % (id_, id_)
            self.app.userdb.cursor.execute(sql)
            if self.app.userdb.cursor.fetchall():
                dialogs.error(_(u"Category is not empty."))
            else:
                sql = "delete from sqllib_cat where id = %s" % id_
                self.app.userdb.cursor.execute(sql)
                self.app.userdb.connection.commit()
                self.refresh()
                self.plugin.rebuild_menues()

    def on_save(self, *args):
        selection = self.list.get_selection()
        model, iter = selection.get_selected()
        if not iter: return
        if model.get_value(iter, 3):
            id_ = model.get_value(iter, 0)
            sql = "update sqllib_sql set title = ?, description = ?, \
            statement = ? where id = ?"
            args= (self.xml.get_widget("sqllib_name").get_text(),
                   self.xml.get_widget("sqllib_description").get_text() or None,
                   self.statement.get_buffer().get_text(*self.statement.get_buffer().get_bounds()),
                   id_
                   )
            self.app.userdb.cursor.execute(sql, args)
            self.app.userdb.connection.commit()
            self.refresh()
            self.plugin.rebuild_menues()
        else:
            id_ = model.get_value(iter, 0)
            sql = "update sqllib_cat set name = ? where id = ?"
            self.app.userdb.cursor.execute(sql,
                                           (self.xml.get_widget("sqllib_name").get_text(),
                                            id_))
            self.app.userdb.connection.commit()
            self.refresh()
            self.plugin.rebuild_menues()

    def on_selection_changed(self, selection):
        model, iter = selection.get_selected()
        if iter and model.get_value(iter, 3):
            id_ = model.get_value(iter, 0)
            sql = "select title, description, statement from sqllib_sql \
            where id = %s" % id_
            self.app.userdb.cursor.execute(sql)
            data = self.app.userdb.cursor.fetchone()
            self.xml.get_widget("sqllib_name").set_text(data[0] or "")
            self.xml.get_widget("sqllib_description").set_text(data[1] or "")
            self.xml.get_widget("sqllib_description").set_sensitive(True)
            self.statement.get_buffer().set_text(data[2] or "")
            self.statement.set_sensitive(True)
            self.xml.get_widget("sqllib_details_table").set_sensitive(True)
        elif iter:
            id_ = model.get_value(iter, 0)
            sql = "select name from sqllib_cat where id = %s" % id_
            self.app.userdb.cursor.execute(sql)
            data = self.app.userdb.cursor.fetchone()
            self.xml.get_widget("sqllib_name").set_text(data[0] or None)
            self.xml.get_widget("sqllib_description").set_text("")
            self.xml.get_widget("sqllib_description").set_sensitive(False)
            self.statement.get_buffer().set_text("")
            self.statement.set_sensitive(False)
            self.xml.get_widget("sqllib_details_table").set_sensitive(True)
        else:
            self.xml.get_widget("sqllib_name").set_text("")
            self.xml.get_widget("sqllib_description").set_text("")
            self.statement.get_buffer().set_text("")
            self.xml.get_widget("sqllib_details_table").set_sensitive(False)

    def on_statement_new(self, menuitem, parent):
        sql = "insert into sqllib_sql (title, category, statement) \
        values (?, ?, 'SELECT * FROM table;')"
        self.app.userdb.cursor.execute(sql, (_(u"New query"), parent))
        id = self.app.userdb.cursor.lastrowid
        self.app.userdb.connection.commit()
        self.plugin.rebuild_menues()
        self.refresh()

    def refresh(self):
        model = self.list.get_model()
        model.clear()
        self.init_items(model)

    def init_items(self, model, parent=None):
        sql = "select id, name from sqllib_cat where parent "
        if parent:
            sql += "= %s" % model.get_value(parent, 0)
        else:
            sql += "is null"
        self.app.userdb.cursor.execute(sql)
        for item in self.app.userdb.cursor.fetchall():
            iter = model.append(parent)
            model.set(iter, 0, item[0], 1, item[1],
                      2, "gtk-open", 3, False,
                      4, item[0])
            sql = "select id, title from sqllib_sql where category = %s" % item[0]
            self.app.userdb.cursor.execute(sql)
            for citem in self.app.userdb.cursor.fetchall():
                citer = model.append(iter)
                model.set(citer, 0, citem[0], 1, citem[1],
                          2, "gtk-edit", 3, True,
                          4, "ZZZZZZZZZZZZZZZZZZZZ%s" % citem[1])
            sql = "select count(*) from sqllib_cat where parent = %s" % item[0]
            self.app.userdb.cursor.execute(sql)
            if self.app.userdb.cursor.fetchone()[0] != 0:
                self.init_items(model, iter)
        if parent == None:
            sql = "select id, title from sqllib_sql where category is null"
            self.app.userdb.cursor.execute(sql)
            for citem in self.app.userdb.cursor.fetchall():
                citer = model.append(None)
                model.set(citer, 0, citem[0], 1, citem[1],
                          2, "gtk-edit", 3, True,
                          4, "ZZZZZZZZZZZZZZZZZZZZ%s" % citem[1])


TABLE_LIB_CAT = """create table sqllib_cat (
    id integer primary key not null,
    name text,
    parent integer references sqllib_cat
);"""

TABLE_LIB_SQL = """create table sqllib_sql (
    id integer primary key not null,
    title text not null,
    description text,
    statement text not null,
    category integer
);"""



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""Custom toolbar"""

from gettext import gettext as _

import gtk
import gobject

from cf.backends import TRANSACTION_IDLE, TRANSACTION_COMMIT_ENABLED
from cf.backends import TRANSACTION_ROLLBACK_ENABLED
from cf.ui import GladeWidget
from cf.ui.widgets import ConnectionButton


class CFToolbar(GladeWidget):

    def __init__(self, instance, xml=None, cb_provider=None):
        GladeWidget.__init__(self, instance.app,
                             xml or instance.xml, "toolbar",
                             cb_provider=cb_provider)
        self.tb_connection = ConnectionButton(self.app, xml or instance.xml)
        tb_open = self.xml.get_widget("tb_open")
        recent_menu = gtk.RecentChooserMenu(self.app.recent_manager)
        filter = gtk.RecentFilter()
        filter.add_mime_type("text/x-sql")
        recent_menu.add_filter(filter)
        recent_menu.set_filter(filter)
        recent_menu.set_show_not_found(False)
        recent_menu.set_sort_type(gtk.RECENT_SORT_MRU)
        recent_menu.connect("item-activated",
                            instance.on_recent_item_activated)
        tb_open.set_menu(recent_menu)
        self._editor = None
        self.__editor_signals = []
        self.__buffer_signals = []
        self.__conn_signals = []
        self.set_editor(None)
        for item in self.widget.get_children():
            if isinstance(item, gtk.ToolItem):
                item.set_is_important(False)
                item.set_homogeneous(False)

    def on_buffer_changed(self, buffer, editor):
        btn_save = self.xml.get_widget("tb_save")
        if not editor or not editor.get_filename():
            btn_save.set_sensitive(False)
            return
        btn_save.set_sensitive(editor.file_contents_changed())

    def on_connection_notify(self, connection, property):
        if property.name == "transaction-state":
            value = connection.get_property(property.name)
            gobject.idle_add(self.set_transaction_state, value)

    def on_editor_connection_changed(self, editor, conn):
        if editor != self._editor:
            return
        self.xml.get_widget("tb_execute").set_sensitive(bool(conn))
        if conn:
            self.set_transaction_state(conn.get_property("transaction-state"))
            self.__conn_signals.append(conn.connect("notify",
                                                    self.on_connection_notify))
        else:
            self.set_transaction_state(None)
        self.tb_connection.set_editor(editor)

    def set_transaction_state(self, value):
        if value == None:
            for item in ("tb_commit", "tb_rollback", "tb_begin"):
                self.xml.get_widget(item).set_sensitive(False)
                return
        self.xml.get_widget("tb_commit").set_sensitive(
            (value & TRANSACTION_COMMIT_ENABLED) != 0)
        self.xml.get_widget("tb_rollback").set_sensitive(
            (value & TRANSACTION_ROLLBACK_ENABLED) != 0)
        self.xml.get_widget("tb_begin").set_sensitive(
            (value & TRANSACTION_IDLE) != 0)

    def set_editor(self, editor):
        while self.__editor_signals:
            self._editor.disconnect(self.__editor_signals.pop())
        if self._editor:
            buffer = self._editor.get_buffer()
            while self.__buffer_signals:
                buffer.disconnect(self.__buffer_signals.pop())
        self._editor = editor
        self.tb_connection.set_editor(editor)
        for item in ["tb_cut", "tb_copy", "tb_paste"]:
            widget = self.xml.get_widget(item)
            if not widget:
                continue
            widget.set_sensitive(bool(editor))
        if self._editor:
            self.__editor_signals.append(
                self._editor.connect("connection-changed",
                                     self.on_editor_connection_changed))
            self.on_editor_connection_changed(self._editor,
                                              self._editor.connection)
            buffer = self._editor.get_buffer()
            self.__buffer_signals.append(
                buffer.connect("changed", self.on_buffer_changed, editor))





# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# For the apachelogs module:
#   (c) Kevin Scott (kevin.scott@gmail.com)
#   modified by bkjones for the Loghetti project (http://code.google.com/p/loghetti/)

# $Id$

"""Backend for http server (Apache, Lighttpd) log files

This backend can be used for simple log file analysis of
HTTP logs in combined format.
"""

import gtk

from cf.plugins.core import DBBackendPlugin, PLUGIN_TYPE_BACKEND
from cf.backends import DBConnectError
from cf.backends.dbapi2helper import DbAPI2Connection
from cf.datasources import DatasourceInfo
from cf.plugins.mixins import MenubarMixin, EditorMixin
from cf.ui.widgets import CustomImageMenuItem
from cf.backends.schema import *

import os
import sys
import md5
from gettext import gettext as _
from tempfile import mkstemp

import logging
log = logging.getLogger("HTTP-LOG")

import apachelogs

class HttpLogBackendPlugin(DBBackendPlugin):
    id = "crunchyfrog.backend.http"
    plugin_type = PLUGIN_TYPE_BACKEND
    name = "HTTP log file backend"
    description = "Analyse HTTP log files with SQL"
    author = "Andi Albrecht"
    license = "GPL"
    homepage = "http://cf.andialbrecht.de"
    version = "0.1"
    
    def __init__(self, *args, **kw):
        DBBackendPlugin.__init__(self, *args, **kw)
        self.db_file = None
        
    @classmethod
    def _get_filename(cls, chooser):
        return chooser.get_filename()
        
    def shutdown(self):
        log.info("Shutting down HTTP log backend")
        if self.db_file and os.path.isfile(self.db_file):
            os.remove(self.db_file)
    
    @classmethod
    def get_datasource_options_widgets(cls, data_widgets, initial_data=None):
        lbl = gtk.Label(_(u"Log file:"))
        lbl.set_alignment(0, 0.5)
        file_chooser = gtk.FileChooserButton(_(u"Select HTTP log file"))
        if initial_data:
            file_chooser.select_filename(initial_data.options.get("log_file"))
        data_widgets["log_file"] = (cls._get_filename, file_chooser)
        return data_widgets, [lbl, file_chooser]
    
    @classmethod
    def get_label(cls, datasource_info):
        return "%s (http-log://%s)" % (datasource_info.name, datasource_info.options.get("log_file"))
    
    def dbconnect(self, data):
        create_db = False
        if not self.db_file:
            create_db = True
            fd, self.db_file = mkstemp(".db", "cf-http-log-")
        try:
            import sqlite3
            if create_db:
                self.init_db(self.db_file, data["log_file"])
            real_conn = sqlite3.connect(self.db_file)
            return HttpLogConnection(self, self.app, real_conn, data["log_file"])
        except StandardError, err:
            raise DBConnectError(str(err))
    
    def test_connection(self, data):
        try:
            conn = self.dbconnect(data)
            conn.close()
        except DBConnectError, err:
            return str(err)
        return None
    
    def init_db(self, dbfile, logfile):
        conn = sqlite3.connect(dbfile)
        cur = conn.cursor()
        cur.execute(TABLE_CREATE)
        print logfile
        lf = apachelogs.ApacheLogFile(logfile)
        for line in lf:
            data = dict()
            for name in dir(line):
                if name.startswith("__"):
                    continue
                data[name] = getattr(line, name)
                if name.startswith("http_response"):
                    data[name] = int(data[name])
            sql = SQL_INSERT % data
            cur.execute(sql)
        conn.commit()
    
class HttpLogConnection(DbAPI2Connection):
    
    def __init__(self, backend, app, real_conn, log_file):
        DbAPI2Connection.__init__(self, backend, app, real_conn)
        self.log_file = log_file
    
    def get_server_info(self):
        return self.log_file
    
    
TABLE_CREATE = """
create table logfile (
    http_method text,
    http_response_code integer,
    http_response_size integer,
    http_user text,
    http_vers text,
    ident text,
    ip text,
    referrer text,
    request_line text,
    time text,
    url text,
    user_agent text
);
"""
SQL_INSERT = """
INSERT INTO log (http_method, http_response_code, http_response_size,
http_user, http_vers, referrer, request_line, time, url, user_agent,
ident, ip)
values ('%(http_method)s', %(http_response_code)d, %(http_response_size)d,
'%(http_user)s', '%(http_vers)s', '%(referrer)s', '%(request_line)s', 
'%(time)s', '%(url)s', '%(user_agent)s', '%(ident)s', '%(ip)s')
"""


try:
    import sqlite3
except ImportError:
    HttpLogBackendPlugin.INIT_ERROR = _(u"Python module sqlite3 required.")


# Copyright (C) 2008 Andi Albrecht, albrecht.andi@gmail.com
#
# This module is part of python-sqlparse and is released under
# the BSD License: http://www.opensource.org/licenses/bsd-license.php.

"""Parse SQL statements."""

import logging
import os

# Setup namespace
from errors import SQLParseError
from errors import SQLParseParseError
from errors import SQLParseMultiStatementError
from query import Query

import tokenizer as _tokenizer
from dialects import DialectDefault, DialectPSQL
_default_dialect = DialectDefault()


if 'SQLPARSE_DEBUG' in os.environ:
    logging.basicConfig(level=logging.DEBUG)


def sqlparse(statement, dialect=None):
    """Parse a SQL statement.

    Args:
        statement: String containing a SQL statement.

    Returns:
        A Query object.

    Raises:
        SQLParseParseError if parsing fails.
        SQLParseMultiStatementError if statement has more than one statement.
    """
    if dialect is None:
        dialect = _default_dialect
    return Query(statement, dialect)


def sqlsplit(statements, dialect=None):
    """Parse statements in a list of statements."""
    if dialect is None:
        dialect = _default_dialect
    splitted = _tokenizer._split_statements(_tokenizer._tokenize(statements,
                                                                 dialect),
                                            dialect)
    return [_tokenizer._join_tokens(tokens)
            for tokens in splitted]



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$
"""Export filter"""

from cf.plugins.core import ExportPlugin

from gettext import gettext as _

class CSVExportFilter(ExportPlugin):
    id = "crunchyfrog.export.csv"
    name = _(u"CSV Export")
    description = _(u"Export query results as CSV files")
    author = "Andi Albrecht"
    license = "GPL"
    homepage = "http://cf.andialbrecht.de"
    version = "0.1"
    has_options = False

    file_filter_name = _(u"Text CSV (.csv)")
    file_filter_pattern = ["*.csv"]

    def __init__(self, app):
        ExportPlugin.__init__(self, app)

    def export(self, description, rows, options):
        import csv
        fp = open(options["filename"], "w")
        w = csv.writer(fp)
        w.writerows(rows)
        fp.close()

class OOCalcExportFilter(ExportPlugin):
    id = "crunchyfrog.export.odc"
    name = _(u"OpenDocument Export")
    description = _(u"Export query results as OpenDocument Spreadsheets")
    author = "Andi Albrecht"
    license = "GPL"
    homepage = "http://cf.andialbrecht.de"
    version = "0.1"

    file_filter_name = _(u"OpenDocument Spreadsheet (.ods)")
    file_filter_pattern = ["*.ods"]

    def __init__(self, app):
        ExportPlugin.__init__(self, app)

    def export(self, description, rows, options):
        from cf.thirdparty import ooolib
        import pwd
        import os
        import types
        doc = ooolib.Calc()
        doc.set_meta('title', _(u"CrunchyFrog Data Export"))
        doc.set_meta('description', options.get("query", ""))
        doc.set_meta('creator', pwd.getpwuid(os.getuid())[4].split(",", 1)[0])
        doc.set_cell_property('bold', True)
        for i in range(len(description)):
            doc.set_cell_value(i+1, 1, "string", description[i][0])
        doc.set_cell_property('bold', False)
        for i in range(len(rows)):
            for j in range(len(rows[i])):
                value = rows[i][j]
                if value == None:
                    continue
                if type(value) == types.FloatType:
                    otype = "float"
                elif type(value) == types.IntType:
                    otype = "int"
                elif type(value) == types.BooleanType:
                    otype = "boolean"
                else:
                    otype = "string"
                doc.set_cell_value(j+1, i+2, otype, value)
        doc.save(options["filename"])



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""Schema browsing base classes"""

import gobject

from gettext import gettext as _

class SchemaProvider(gobject.GObject):

    def __init__(self):
        self.__gobject_init__()

    def fetch_children(self, connection, parent):
        return []

    def get_details(self, connection, obj):
        return None

class Node(gobject.GObject):
    name = None
    description = None
    icon = "gtk-missing-image"
    has_children = True
    has_details = False

    def __init__(self, name=None, description=None, *args, **kwargs):
        self.__gobject_init__()
        if name:
            self.name = name
        self.description = description
        if kwargs.has_key("has_details"):
            self.has_details = kwargs.pop("has_details")
        for key, value in kwargs.items():
            self.set_data(key, value)

class Collection(Node):
    icon = "gtk-open"

class SchemaCollection(Collection):
    name = _(u"Schematas")

class Schema(Node):
    name = _(u"Schema")
    icon = "gtk-open"

class TableCollection(Collection):
    name = _(u"Tables")
    icon = "stock_data-tables"

class Table(Node):
    name = _(u"Table")
    icon = "stock_data-table"

class ColumnCollection(Collection):
    name = _(u"Columns")

class Column(Node):
    name = _(u"Column")
    icon = "stock_insert-columns"
    has_children = False

class ViewCollection(Collection):
    name = _(u"Views")
    icon = "stock_data-tables"

class View(Node):
    name = _(u"View")
    icon = "stock_data-table"

class SequenceCollection(Collection):
    name = _(u"Sequences")
    icon = "stock_sort-column-ascending"

class Sequence(Node):
    name = _(u"Sequence")
    icon = "stock_sort-row-ascending"
    has_children = False

class FunctionCollection(Collection):
    name = _(u"Functions")
    icon = "stock_script"

class Function(Node):
    name = _(u"Function")
    icon = "stock_macro-check-brackets"
    has_children = False

class PackageCollection(Collection):
    name = _(u"Packages")

class Package(Node):
    name = _(u"Package")
    icon = "gnome-mime-application-x-archive"

class ConstraintCollection(Collection):
    name = _(u"Constraints")

class Constraint(Node):
    name = _(u"Contraint")
    icon = "gtk-spell-check"

class IndexCollection(Collection):
    name = _(u"Indexes")

class Index(Node):
    name = _(u"Index")
    icon = "stock_navigator-indexes"
    has_children = False



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""Release information"""

name = "CrunchyFrog"
appname = name.lower()
version = "0.3.0"
copyright = "Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>"
description = "Database front-end for GNOME"
author = "Andi Albrecht"
author_email = "albrecht.andi@gmail.com"
url = "http://cf.andialbrecht.de"
long_description = """CrunchyFrog is a database front-end for the GNOME desktop.

Supported databases
===================

 * PostgreSQL (requires psycopg2)
 * MySQL (requires MySQLdb)
 * SQLite3 (requires python-sqlite3)
 * Oracle (requires cx_Oracle)
 * MS-SQL Server (requires pymssql)
 * LDAP (requires python-ldap)

It is written in Python (PyGTK) and can be extended using plugins.

Check out the homepage for the most recent version and details."""

license = """This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>."""

translators="""Launchpad Contributions:
  Kari Argillander (https://launchpad.net/~kari-argillander)
  Edwin Dijk (https://launchpad.net/~sitix)
  Anthony Guéchoum (https://launchpad.net/~athael)
  Guybrush88 (https://launchpad.net/~erpizzo)
  Ryutatsu Ishigami (https://launchpad.net/~ai-inside+ubuntu)
  Luca Livraghi (https://launchpad.net/~luca91)
  Marion Cisternas Milla (https://launchpad.net/~mcisternas)
  Christian Olate Opazo (https://launchpad.net/~cristian-olate)
  Penegal (https://launchpad.net/~penegal)
  Nicola Piovesan (https://launchpad.net/~piovesannicola)
  Enrico Rosina (https://launchpad.net/~e-rosina)
  Xoan Sampaiño (https://launchpad.net/~xoan)
  Skiros (https://launchpad.net/~magoskiros)
  Pierre Slamich (https://launchpad.net/~pierre-slamich)
  Jeppe Toustrup (https://launchpad.net/~tenzer)
  ubby (https://launchpad.net/~kostas-sytske)
  Linus C Unneback (https://launchpad.net/~linus-folkdatorn)
  Yaron (https://launchpad.net/~sh-yaron)
  Muhammad Zulfikar (https://launchpad.net/~zulfikars)
  0x00080h (https://launchpad.net/~einstein89)
"""
